# utils.py
import os
import shutil
from pathlib import Path
from loguru import logger
import yaml

def load_config(file_path, default_path):
    if not os.path.exists(file_path):
        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        shutil.copy(default_path, file_path)
        logger.info("設定ファイルがテンプレートから作成されました：{}", file_path)
    with open(file_path, "r", encoding='utf8') as file:
        return yaml.safe_load(file)

def load_file_content(user_file_path, default_file_path):
    user_path = Path(user_file_path)
    base_path = Path(__file__).parent
    default_path = base_path / default_file_path
    logger.debug("デフォルトパス：{}", default_path)
    logger.debug("ユーザーパス：{}... 存在する：{}", user_path, os.path.exists(user_path))
    if not os.path.exists(user_path):
        os.makedirs(user_path.parent, exist_ok=True)
        shutil.copy(default_path, user_path)
        logger.info("テンプレートファイルがこちらにコピーされました：{}", user_path)
    with open(user_path, "r", encoding="utf8") as file:
        return file.read()

def copy_cicd_file_if_missing(config, output_dir):
    cicd_file_path = Path(output_dir) / config['harmon_ai']['product'].get('cicd_file_path', 'publish-to-pypi.yml')
    cicd_main_path = Path(config["harmon_ai"]["main"]["main_dir"]) /  Path(config["harmon_ai"]["product"]["github_cicd_dir"]) / config['harmon_ai']['product'].get('cicd_main_path', 'publish-to-pypi.yml')
    template_path = Path(__file__).parent / config['harmon_ai']['product'].get('cicd_template_path', 'templates/publish-to-pypi.yml')
    if not cicd_file_path.exists():
        shutil.copy(template_path, cicd_file_path)
        logger.info("[Development] CICDファイルがテンプレートからコピーされました：{}", cicd_file_path)

        # ファイルを読み込んで内容を置換
        with open(cicd_file_path, 'r+', encoding='utf-8') as file:
            content = file.read()
            content = content.replace("https://pypi.org/p/harmon_ai", f"https://pypi.org/p/{config['harmon_ai']['environment']['package_name']}")
            file.seek(0)
            file.write(content)
            file.truncate()  # 既存の内容が新しい内容より長い場合、余分な部分を削除
            logger.info(f"[Development] URLが置換されました：{cicd_file_path}")

    if not cicd_main_path.exists():
        os.makedirs(cicd_main_path.parent, exist_ok=True)
        shutil.copy(cicd_file_path, cicd_main_path)
        logger.info("[Production] CICDファイルが開発用からコピーされました：{}", cicd_main_path)

def config_preview(config):
    for key, value in config['harmon_ai']['environment'].items():
        logger.info("{}：{}", key.replace('_', ' ').capitalize(), value)


import os
from pathlib import Path
from .utils import load_config, load_file_content, copy_cicd_file_if_missing, config_preview
from art import tprint

class HarmonAI:
    def __init__(self):
        config_path = os.path.expanduser("./.harmon_ai/config.yml")
        self.config = load_config(config_path, os.path.join(Path(__file__).parent, "templates/config.example.yml"))

        tprint("-- HarmonAI --")
        config_preview(self.config)

        output_dir = self.config['harmon_ai']['development']['output_dir']
        os.makedirs(output_dir, exist_ok=True)

        self.important_message = load_file_content(
            os.path.join(output_dir, self.config['harmon_ai']['product']['important_message_file']),
            "templates/important_template.md"
        )
        self.sections_content = load_file_content(
            os.path.join(output_dir, self.config['harmon_ai']['product']['sections_content_file']),
            "templates/sections_template.md"
        )

        # CICDファイルを確認してコピー
        copy_cicd_file_if_missing(self.config, output_dir)

    @staticmethod
    def load_template(template_name):
        """指定されたテンプレート名でテンプレートファイルを読み込む"""
        template_path = os.path.join(os.path.dirname(__file__), "templates", f"{template_name}.md")
        with open(template_path, "r", encoding="utf8") as file:
            return file.read()

    def run(self):
        template = self.load_template("readme_template")
        badge_template = self.generate_badges()
        sns_badges = self.generate_sns_badges()

        template = template.replace("{repo_name}", self.config['harmon_ai']['environment']['repo_name'])
        template = template.replace("{owner_name}", self.config['harmon_ai']['environment']['owner_name'])
        template = template.replace("{package_name}", self.config['harmon_ai']['environment']['package_name'])
        template = template.replace("{icon_url}", self.config['harmon_ai']['environment']['icon_url'])
        template = template.replace("{title}", self.config['harmon_ai']['environment']['title'])
        template = template.replace("{subtitle}", self.config['harmon_ai']['environment']['subtitle'])
        template = template.replace("{badges}", badge_template)
        template = template.replace("{important_message}", self.important_message)
        template = template.replace("{sections_content}", self.sections_content)
        template = template.replace("{sns_badges}", sns_badges)

        output_path = os.path.join(self.config['harmon_ai']['development']['output_dir'], self.config['harmon_ai']['product']['output_file'])
        with open(output_path, "w", encoding="utf8") as file:
            file.write(template)

    def generate_badges(self):
        template = self.load_template("badges_template")
        template = template.replace("{repo_name}", self.config['harmon_ai']['environment']['repo_name'])
        template = template.replace("{owner_name}", self.config['harmon_ai']['environment']['owner_name'])
        template = template.replace("{package_name}", self.config['harmon_ai']['environment']['package_name'])
        return template

    def generate_sns_badges(self):
        template = self.load_template("sns_template")
        template = template.replace("{website_url}", self.config['harmon_ai']['environment']['website_url'])
        template = template.replace("{github_url}", self.config['harmon_ai']['environment']['github_url'])
        template = template.replace("{twitter_url}", self.config['harmon_ai']['environment']['twitter_url'])
        template = template.replace("{blog_url}", self.config['harmon_ai']['environment']['blog_url'])
        return template


# main.py
import sys
from .harmon_ai import HarmonAI
from loguru import logger
import os

logger.configure(
    handlers=[
        {
            "sink": sys.stderr,
            "format": "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | <level>{level:<8}</level> | <cyan>{name:<45}:{line:<5}</cyan> | <level>{message}</level>",
            "colorize": True,
        }
    ]
)

def main():
    harmon_ai = HarmonAI()
    harmon_ai.run()
    logger.success("READMEファイルが生成されました：{}", os.path.join(harmon_ai.config['harmon_ai']['development']['output_dir'], harmon_ai.config['harmon_ai']['product']['output_file']))

if __name__ == "__main__":
    main()